// The MIT License

// Copyright (c) 2010 Massachusetts Institute of Technology

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

// Author: Richard Uhler ruhler@mit.edu

import Connectable::*;
import GetPut::*;
import FIFO::*;
import RegFile::*;

import DDR2Types::*;

// Simulated DDR2 which can be connected to a DDR2Client.

// We simulate the DDR2 with a reg file.
// Bluesim is clever enough to support this.
// We assume 64 bit word addressed. All of the least significant bits are
// used. The most significant bits may be ignored.
typedef Bit#(31) DataIndex;
function DataIndex getIndex(DDR2Address addr) = truncate(addr);

// This implementation uses burst 4 like the DDR2 mig interface. Each write
// command must be accompanied by 4 64 bit data words to write (they will be
// written consecutively). Each read command reads 4 64 bit words.
module mkDDR2Simulator(DDR2Server);
    FIFO#(DDR2ReqCmd) cmds <- mkFIFO();
    FIFO#(DDR2ReqData) wdata <- mkFIFO();
    FIFO#(DDR2Data) rdata <- mkFIFO();

    // How many of the words for the current command we have processed so far.
    Reg#(Bit#(2)) count <- mkReg(0);

    RegFile#(DataIndex, DDR2Data) data <- mkRegFileFull();
    
    rule doread(cmds.first().rnw);
        rdata.enq(data.sub(getIndex(cmds.first().addr + zeroExtend(count))));

        if (count == maxBound) begin
            // We're done with this command.
            cmds.deq();
            count <= 0;
        end else begin
            count <= count+1;
        end
    endrule

    rule dowrite(!cmds.first().rnw);

        wdata.deq();

        DataIndex idx = getIndex(cmds.first().addr+zeroExtend(count));
        DDR2Data word = data.sub(idx);

        // Only update those bytes for which byte enable is set
        for (Integer i = 0; i < 8; i = i+1) begin
            if (wdata.first().be[0] == 1) begin
                Bit#(8) byte_ = wdata.first().data[8*i+7:8*i];
                word[8*i+7:8*i] = byte_;
            end
        end

        data.upd(idx, word);

        if (count == maxBound) begin
            cmds.deq();
            count <= 0;
        end else begin
            count <= count+1;
        end
    endrule

    interface DDR2RequestServer request;
        interface Put command = toPut(cmds);
        interface Put data = toPut(wdata);
    endinterface

    interface Get response = toGet(rdata);

endmodule

