// The MIT License

// Copyright (c) 2010 Massachusetts Institute of Technology

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

// Author: Richard Uhler ruhler@mit.edu

module mig_33_wrapper
(
    // DDR2 wires
    inout   [63:0]  ddr2_dq,
    output  [12:0]  ddr2_a,
    output  [1:0]   ddr2_ba,
    output          ddr2_ras_n,
    output          ddr2_cas_n,
    output          ddr2_we_n,
    output  [0:0]   ddr2_cs_n,
    output  [0:0]   ddr2_odt,
    output  [0:0]   ddr2_cke,
    output  [7:0]   ddr2_dm,
    inout   [7:0]   ddr2_dqs,
    inout   [7:0]   ddr2_dqs_n,
    output  [1:0]   ddr2_ck,
    output  [1:0]   ddr2_ck_n,

    // Clock inputs
    input           clk0,
    input           clk90,
    input           clkdiv0,
    input           clk200,
    input           sys_rst_n,
    input           locked,

    // Clock output
    output          rst0_tb_n,
    output          clk0_tb,

    // af method
    input           app_af_enable,
    output          app_af_ready,
    input   [2:0]   app_af_cmd,
    input   [30:0]  app_af_addr,

    // wdf method
    input           app_wdf_enable,
    output          app_wdf_ready,
    input   [127:0] app_wdf_data,
    input   [15:0]  app_wdf_mask_data,

    // data method
    output          rd_data_ready,
    output  [127:0] rd_data_out
);

    wire app_af_afull;
    wire app_wdf_afull;
    wire rd_data_valid;
    wire phy_init_done;

    assign app_af_ready = (~app_af_afull) & phy_init_done;
    assign app_wdf_ready = (~app_wdf_afull) & phy_init_done;
    assign rd_data_ready = rd_data_valid & phy_init_done;

    wire app_af_wren;
    wire app_wdf_wren;
    wire [127:0] rd_data_fifo_out;

    assign app_af_wren = app_af_enable;
    assign app_wdf_wren = app_wdf_enable;
    assign rd_data_out = rd_data_fifo_out;

    wire rst0_tb;                   
    assign rst0_tb_n = ~rst0_tb;

 mig_33 # (
    .BANK_WIDTH(2),   
                                       // # of memory bank addr bits.
    .CKE_WIDTH(1),   
                                       // # of memory clock enable outputs.
    .CLK_WIDTH(2),   
                                       // # of clock outputs.
    .COL_WIDTH(10),   
                                       // # of memory column bits.
    .CS_NUM(1),   
                                       // # of separate memory chip selects.
    .CS_WIDTH(1),   
                                       // # of total memory chip selects.
    .CS_BITS(0),   
                                       // set to log2(CS_NUM) (rounded up).
    .DM_WIDTH(8),   
                                       // # of data mask bits.
    .DQ_WIDTH(64),   
                                       // # of data width.
    .DQ_PER_DQS(8),   
                                       // # of DQ data bits per strobe.
    .DQS_WIDTH(8),   
                                       // # of DQS strobes.
    .DQ_BITS(6),   
                                       // set to log2(DQS_WIDTH*DQ_PER_DQS).
    .DQS_BITS(3),   
                                       // set to log2(DQS_WIDTH).
    .ODT_WIDTH(1),   
                                       // # of memory on-die term enables.
    .ROW_WIDTH(13),   
                                       // # of memory row and # of addr bits.
    .ADDITIVE_LAT(0),   
                                       // additive write latency.
    .BURST_LEN(4),   
                                       // burst length (in double words).
    .BURST_TYPE(0),   
                                       // burst type (=0 seq; =1 interleaved).
    .CAS_LAT(3),   
                                       // CAS latency.
    .ECC_ENABLE(0),   
                                       // enable ECC (=1 enable).
    .APPDATA_WIDTH(128),   
                                       // # of usr read/write data bus bits.
    .MULTI_BANK_EN(1),   
                                       // Keeps multiple banks open. (= 1 enable).
    .TWO_T_TIME_EN(1),   
                                       // 2t timing for unbuffered dimms.
    .ODT_TYPE(1),   
                                       // ODT (=0(none),=1(75),=2(150),=3(50)).
    .REDUCE_DRV(0),   
                                       // reduced strength mem I/O (=1 yes).
    .REG_ENABLE(0),   
                                       // registered addr/ctrl (=1 yes).
    .TREFI_NS(7800),   
                                       // auto refresh interval (ns).
    .TRAS(40000),   
                                       // active->precharge delay.
    .TRCD(15000),   
                                       // active->read/write delay.
    .TRFC(105000),   
                                       // refresh->refresh, refresh->active delay.
    .TRP(15000),   
                                       // precharge->command delay.
    .TRTP(7500),   
                                       // read->precharge delay.
    .TWR(15000),   
                                       // used to determine write->precharge.
    .TWTR(7500),   
                                       // write->read delay.
    .HIGH_PERFORMANCE_MODE("TRUE"),   
                              // # = TRUE, the IODELAY performance mode is set
                              // to high.
                              // # = FALSE, the IODELAY performance mode is set
                              // to low.
    .SIM_ONLY(0),   
                                       // = 1 to skip SDRAM power up delay.
    .DEBUG_EN(0),   
                                       // Enable debug signals/controls.
                                       // When this parameter is changed from 0 to 1,
                                       // make sure to uncomment the coregen commands
                                       // in ise_flow.bat or create_ise.bat files in
                                       // par folder.
    .CLK_PERIOD(5000),   
                                       // Core/Memory clock period (in ps).
    .RST_ACT_LOW(1)     
                                       // =1 for active low reset, =0 for active high.
)
u_mig_33 (
    .ddr2_dq                   (ddr2_dq),
    .ddr2_a                    (ddr2_a),
    .ddr2_ba                   (ddr2_ba),
    .ddr2_ras_n                (ddr2_ras_n),
    .ddr2_cas_n                (ddr2_cas_n),
    .ddr2_we_n                 (ddr2_we_n),
    .ddr2_cs_n                 (ddr2_cs_n),
    .ddr2_odt                  (ddr2_odt),
    .ddr2_cke                  (ddr2_cke),
    .ddr2_dm                   (ddr2_dm),
    .sys_rst_n                 (sys_rst_n),
    .phy_init_done             (phy_init_done),
    .locked                    (locked),
    .rst0_tb                   (rst0_tb),
    .clk0                      (clk0),
    .clk0_tb                   (clk0_tb),
    .clk90                     (clk90),
    .clkdiv0                   (clkdiv0),
    .clk200                    (clk200),
    .app_wdf_afull             (app_wdf_afull),
    .app_af_afull              (app_af_afull),
    .rd_data_valid             (rd_data_valid),
    .app_wdf_wren              (app_wdf_wren),
    .app_af_wren               (app_af_wren),
    .app_af_addr               (app_af_addr),
    .app_af_cmd                (app_af_cmd),
    .rd_data_fifo_out          (rd_data_fifo_out),
    .app_wdf_data              (app_wdf_data),
    .app_wdf_mask_data         (app_wdf_mask_data),
    .ddr2_dqs                  (ddr2_dqs),
    .ddr2_dqs_n                (ddr2_dqs_n),
    .ddr2_ck                   (ddr2_ck),
    .ddr2_ck_n                 (ddr2_ck_n)
);


endmodule

