// The MIT License

// Copyright (c) 2010 Massachusetts Institute of Technology

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

// Author: Richard Uhler ruhler@mit.edu

import SceMi::*;
import SceMiLayer::*;

// Setup for SCE-MI over PCIE to a Virtex5
import Xilinx::*;
import XilinxPCIE::*;
import Clocks::*;
import DefaultValue::*;
import ClockGeneratorFixed::*;

// DDR2 Stuff
import Connectable::*;
import DDR2Types::*;
import DDR2::*;
import mig_33_wrapper::*;

SceMiLinkType lt = PCIE_VIRTEX5;

// We need to get access to the uncontrolled clock and reset to hook up the
// ddr2. This is a hack to get at that info.
interface DDR2SceMiLayerIfc;
    interface DDR2Client ddr2;
    interface Clock uclock;
    interface Reset ureset;
endinterface

module [SceMiModule] mkDDR2SceMiLayerWrapper(DDR2SceMiLayerIfc);

    DDR2Client m <- mkSceMiLayer();
    Clock uclk <- sceMiGetUClock;
    Reset urst <- sceMiGetUReset;

    interface ddr2 = m;
    interface uclock = uclk;
    interface ureset = urst;
endmodule

interface ML50x_FPGA_DDR2;
    interface ML50x_FPGA ml50x;
    interface DDR2Wires ddr2wires;
endinterface

(* synthesize, no_default_clock, no_default_reset *)
module mkBridge #(Clock pci_sys_clk_p, Clock pci_sys_clk_n,
		  Clock refclk_100, Clock clk200, Reset pci_sys_reset_n)
                 (ML50x_FPGA_DDR2);

    // We want the following clocks derived from refclk_100:
    //  clk0 - 200 MHz
    //  clk90 - 200 MHz with 90 degree phase shift
    //  clkdiv0 - 100 MHz clock
    // All three go to the DDR2
    // We use clkdiv0 as the ref_clk for PCIE.
    ClockGeneratorParams clk_params = defaultValue();
    clk_params.feedback_mul = 8;
    clk_params.clk0_div = 4;
    clk_params.clk1_div = 4;
    clk_params.clk1_phase = 90.000;
    clk_params.clk2_div = 8;

    ClockGenerator clk_gen <- mkClockGeneratorFixed(clk_params, clocked_by refclk_100, reset_by pci_sys_reset_n);

    SceMiV5PCIEArgs pcie_args;
    pcie_args.pci_sys_clk_p = pci_sys_clk_p;
    pcie_args.pci_sys_clk_n = pci_sys_clk_n;
    pcie_args.pci_sys_reset = pci_sys_reset_n;
    pcie_args.ref_clk       = clk_gen.clkout2;
    pcie_args.link_type     = lt;

    (* doc = "synthesis attribute buffer_type of scemi_pcie_ep_trn_clk is \"none\"" *)
    (* doc = "synthesis attribute keep of scemi_pcie_ep_trn2_clk is \"true\";" *)
    (* doc = "synthesis attribute clock_signal of scemi_sys_clk_buf is \"yes\";" *)
    SceMiV5PCIEIfc#(DDR2SceMiLayerIfc, 1) scemi <- buildSceMi(mkDDR2SceMiLayerWrapper, pcie_args);

    ReadOnly#(Bool) _isLinkUp         <- mkNullCrossing(noClock, scemi.isLinkUp);
    ReadOnly#(Bool) _isOutOfReset     <- mkNullCrossing(noClock, scemi.isOutOfReset);
    ReadOnly#(Bool) _isClockAdvancing <- mkNullCrossing(noClock, scemi.isClockAdvancing);

    // DDR2
    // The DDR2 controller runs on the uncontrolled clock.
    Reset ddr2rst <- mkAsyncReset(50, pci_sys_reset_n, clk_gen.clkout0);
    DDR2 ddr2 <- mkDDR2Burst4(
        clk_gen.clkout0, clk_gen.clkout1, clk_gen.clkout2,
        clk200, ddr2rst, clk_gen.locked,
        clocked_by scemi.orig_ifc.uclock, reset_by scemi.orig_ifc.ureset);
    mkConnection(scemi.orig_ifc.ddr2, ddr2.server);

    interface ML50x_FPGA ml50x;
        interface pcie = scemi.pcie;

        method leds = zeroExtend({pack(_isClockAdvancing)
                    ,pack(_isOutOfReset)
                    ,pack(_isLinkUp)
                    });
    endinterface

    interface ddr2wires = ddr2.wires;

endmodule: mkBridge

