// The MIT License

// Copyright (c) 2010 Massachusetts Institute of Technology

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

// Author: Richard Uhler ruhler@mit.edu

import FIFO::*;
import GetPut::*;

import DDR2Types::*;
import DDR2FakeBurst::*;

// Sample Dut which interacts with DDR2
// 
// Our sample dut must implement the DDR2Client to interact with DDR2, just
// like any dut that interacts with the DDR2.
// 
// The application our dut will implement is a memory, not to be confused with
// the DDR2. Out dut uses the DDR2 to implement the memory.

typedef Bit#(32) DutAddr;
typedef Bit#(64) DutData;
typedef struct {
    DutAddr addr;
    DutData data;
} DutWrite deriving(Bits, Eq);

interface Dut;
    // Application interface: a simple memory
    interface Put#(DutAddr) readreq;
    interface Get#(DutData) readresp;
    interface Put#(DutWrite) write;

    // DDR2Client interface. All duts which want to use DDR2 on FPGA must
    // implement this interface.
    interface DDR2Client ddr2;
endinterface

module [Module] mkDut(Dut);

    FIFO#(DDR2ReqCmd) cmds <- mkFIFO();
    FIFO#(DDR2ReqData) wdata <- mkFIFO();
    FIFO#(DDR2Data) rdata <- mkFIFO();

    DDR2Client ddr2_nb = interface DDR2Client;
        interface DDR2RequestClient request;
            interface Get command = toGet(cmds);
            interface Get data = toGet(wdata);
        endinterface

        interface Put response = toPut(rdata);
    endinterface;

    DDR2Client ddr2_b <- mkDDR2FakeBurst(ddr2_nb);

    // Write a complete 64 bit word to memory
    interface Put write;
        method Action put(DutWrite w);
            cmds.enq(DDR2ReqCmd { rnw: False, addr: w.addr});
            wdata.enq(DDR2ReqData { be: 8'hFF, data: w.data });
        endmethod
    endinterface

    // Initiate a read request
    interface Put readreq;
        method Action put(DDR2Address addr);
            cmds.enq(DDR2ReqCmd { rnw: True, addr: addr});
        endmethod
    endinterface

    // Get the read response
    interface Get readresp;
        method ActionValue#(DDR2Data) get();
            rdata.deq();    
            return rdata.first();
        endmethod
    endinterface

    interface ddr2 = ddr2_b;

endmodule

