// The MIT License

// Copyright (c) 2010 Massachusetts Institute of Technology

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

// Author: Richard Uhler ruhler@mit.edu

#include <cassert>
#include <string>

#include "TestBench.h"

TestBench::TestBench()
    : m_params("scemi.params"),
      m_scemi(SceMi::Init(SceMi::Version(SCEMI_VERSION_STRING), &m_params)),
      m_readreq("", "scemi_m_readreq_inport", m_scemi),
      m_readresp("", "scemi_m_readresp_outport", m_scemi),
      m_write("", "scemi_m_write_inport", m_scemi),
      m_shutdown("", "scemi_m_shutdown", m_scemi)
{
    m_sthread = new SceMiServiceThread(m_scemi);
}

TestBench::~TestBench()
{
    m_shutdown.blocking_send_finish();
    m_sthread->stop();
    m_sthread->join();
    SceMi::Shutdown(m_scemi);
    delete m_sthread;
}

void TestBench::run()
{
    // We accept three different commands:
    // read addr: reads data from memory and prints the value read
    // write addr data: write data to memory.
    // quit: quit

    std::string cmd;
    bool done = false;
    while (std::cin && !done) {
        std::cout << "$ ";
        std::cin >> cmd;

        if (cmd == "quit") {
            done = true;
        } else if (cmd == "read") {
            unsigned int uiaddr;
            std::cin >> uiaddr;
            DutAddr addr(uiaddr);

            std::cout << "Reading from addr " << addr << "..." << std::endl;
            m_readreq.sendMessage(addr);
            std::cout << "Read req sent, waiting for response..." << std::endl;
            std::cout << m_readresp.getMessage() << std::endl;
        } else if (cmd == "write") {
            unsigned int uiaddr;
            std::cin >> uiaddr;
            DutAddr addr(uiaddr);

            unsigned long uldata;
            std::cin >> uldata;
            DutData data(uldata);

            DutWrite wreq;
            wreq.m_addr = addr;
            wreq.m_data = data;

            m_write.sendMessage(wreq);
            std::cout << "Wrote data " << data << " to addr " << addr << std::endl;
        } else if (cmd == "help") {
            std::cout << "read <addr>" << std::endl;
            std::cout << "\t- Read data at addr" << std::endl;
            std::cout << "write <addr> <data>" << std::endl;
            std::cout << "\t- Write data to addr" << std::endl;
            std::cout << "quit" << std::endl;
            std::cout << "\t- quit" << std::endl;
        } else {
            std::cout << cmd << ": invalid command" << std::endl;
        }
    }
}

int main(int argc, char* argv[])
{
    TestBench tb;
    tb.run();
}

